;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; SMW Line-Guided Rope (sprite 64), by imamelia
;;
;; This is a disassembly of sprite 64 in SMW, the line-guided rope.
;;
;; Uses first extra bit: YES
;;
;; If the first extra bit is clear, the rope will be short, and if the first extra bit is
;; set, the rope will be long.  But you need to use the No Sprite Tile Limits patch,
;; or the rope will always be short.  (Even sprite memory setting 1, which was used
;; with the original, doesn't work properly.)
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

motor_tilemap:
        db $C0,$C2,$E0,$C2
rope_tilemap:
        db $EA,$EA,$EA,$EA,$EA,$EA,$EA,$EA,$EA

!bottom_rope_tile       = $EB
!motor_props            = $37
!rope_props             = $31

print "INIT ",pc
        PHB
        PHK
        PLB
        LDA !7FAB10,x
        AND #$04
        BEQ .short
        INC !1510,x
.short
        INC !1540,x
        JSR horizontal_rope
        JSR horizontal_rope
        INC !1626,x
        PLB
        RTL

print "MAIN ",pc
        PHB
        PHK
        PLB
        JSR horizontal_rope
        PLB
        RTL

horizontal_rope:
        TXA
        ASL #2
        EOR $14
        STA $02                 ;   set the index for the smoke
        AND #$07                ;\  every so many frames...
        ORA $9D                 ; | if sprites are not locked...
        BNE .no_smoke           ;/  make smoke come from the sprite's motor

        LDA $02
        LSR #3
        AND #$01                ;   2 possible X offsets for the smoke
        TAY
        LDA smoke_x_offset,y    ;   set the X displacement of the smoke
        STA $00
        LDA #$F2                ;   Y displacement = $F2
        STA $01
        JSR draw_smoke          ;   show the smoke

.no_smoke
        LDA $13
        AND #$07                ;\  every 8 frames...
        ORA !1626,x             ; | if the sprite is moving...
        ORA $9D                 ;/  and sprites are not locked...
        BNE +
        LDA #$04                ;\  play the ticking sound effect
        STA $1DFA|!Base2        ;/

+       JSR draw_sprite

        LDA !D8,x;
        PHA                     ;\  preserve the low bytes of the sprite's position
        LDA !E4,x               ;/  (why not the high bytes?...)
        PHA
        JSR lg_main
        PLA
        SEC
        SBC !E4,x
        EOR #$FF
        INC
        STA $185E|!Base2
        PLA
        SEC
        SBC !D8,x
        EOR #$FF
        INC
        STA $18B6|!Base2
        LDA $77
        AND #$03
        BNE return

;   interaction code adapted for custom clipping
        LDA $71
        CMP #$01
        BCS no_interact
        LDA #$00
        BIT $0D9B
        BVS +
        LDA $13F9
        EOR $1632,x
+       BNE no_interact

        LDY #$74
        LDA !7FAB10,x
        AND #$04
        BNE +
        LDY #$34
+       STY $06
        LDA #$02
        STA $07
        LDA !E4,x
        CLC
        ADC #$10
        STA $04
        LDA !14E0,x
        ADC #$00
        STA $0A
        LDA !D8,x
        STA $05
        LDA !14D4,x
        STA $0B

        JSL $03B664|!BankB
        JSL $03B72B|!BankB
        BCS interact

no_interact:
        LDA !163E,x             ;   unless the grab timer is already zero...
        BEQ return
        STZ !163E,x             ;   clear it
        STZ $18BE|!Base2        ;   and clear the climbing flag as well
        STZ $74
return:
        RTS

interact:
        LDA !14C8,x             ;\  if the sprite is nonexistent...
        BEQ .not_check_interact ;/  skip the next part of code

        LDA $1470|!Base2        ;\  if the player is carrying something...
        ORA $187A|!Base2        ; | or on Yoshi...
        BNE no_interact         ;/  don't let him/her climb the rope

        LDA #$03                ;\  set the climb timer
        STA !163E,x             ;/

        LDA !154C,x             ;\  if interaction is disabled...
        BNE return              ;/  return

        LDA $18BE|!Base2        ;\  if the player is already climbing...
        BNE .skip_button_check  ;/  don't check to see if he/she is pressing the up button

        LDA $15                 ;\
        AND #$0B                ; | if the player is pressing Up...
        BEQ return              ;/
        STA $18BE|!Base2        ;   set the climbing flag

.skip_button_check
        BIT $16                 ;\ if the player is pressing A or B...
        BPL .dont_jump_off      ;/
        LDA #$B0                ;\ make him/her go upward
        STA $7D                 ;/

.not_check_interact
        STZ $18BE|!Base2        ;   clear the climb flag
        STZ $74
        LDA #$10
        STA !154C,x             ;   and disable interaction for a few frames

.dont_jump_off
        LDA #$80
        TSB $74
        LDY #$00                ; Y = 00
        LDA $185E|!Base2        ; check the necessary position offset
        BPL +                   ; if it is negative...
        DEY                     ; Y = FF
+       CLC
        ADC $94                 ; add the amount the sprite moved on the Y-axis during the line-guide routine
        STA $94                 ; to the player's Y position
        TYA                     ;
        ADC $95                 ; handle the high byte
        STA $95                 ;

        LDA !14E0,x             ; into adjacent scratch RAM
        STA $01
        LDA !E4,x               ;
        STA $00
        REP #$20                ; set A to 16-bit mode
        LDA $94
        SEC
        SBC $00
        CMP #$0000              ; I'm not exactly sure what the purpose of this is...
        BPL +
        INC $94
+       SEP #$20

        LDA $18B6|!Base2        ; the amount the sprite moved on the X-axis during the line-guide routine
        JSR offset_player_y

        LDA !D8,x       ;
        SEC             ;
        SBC #$18        ; sprite X position - $18
        CMP $96         ;
        BEQ .move       ; if the player is at the sprite's X position, there is no need to offset his/her position
        BPL +           ; if the result was positive, do another check with A = 01
        LDA #$FF        ; if the result was positive, do another check with A = FF
        BRA ++
+       LDA #$01
++      JSR offset_player_y
        LDA !1626,x;
.move
        LDA !1626,x     ; if the stationary flag is not already clear...
        BEQ +
        STZ !1626,x     ; clear it
        STZ !1540,x     ; as well as the move timer
+       RTS

offset_player_y:
        LDY #$00        ; Y = 00
        CMP #$00        ; check the position offset
        BPL +           ; if it is negative...
        DEY             ; Y = FF
+       CLC             ;\
        ADC $96         ; | add the amount the sprite moved on the X-axis during the line-guide routine
        STA $96         ;/  to the player's X position
        TYA             ;\
        ADC $97         ; | handle the high byte
        STA $97         ;/
        RTS


;------------------

smoke_x_offset:
        db $F8,$00

draw_smoke:
        LDA !15A0,x             ; if the sprite is offscreen
        ORA !186C,x             ; in either direction...
        BNE .return             ; don't make smoke
        LDY #$03                ; 4 indexes per smoke table
.loop
        LDA $17C0|!Base2,y      ; if this slot is free...
        BEQ .free_slot          ; use it
        DEY                     ; if not, check the next index
        BPL .loop
        RTS

.free_slot
        LDA #$03                ; set the type of smoke
        STA $17C0|!Base2,y      ;
        LDA !E4,x               ;
        ADC $00                 ; smoke X offset
        STA $17C8|!Base2,y      ;
        LDA !D8,x               ;
        ADC $01                 ; smoke Y offset
        STA $17C4|!Base2,y      ;
        LDA #$13                ; time to show smoke
        STA $17CC|!Base2,y      ;
.return
        RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; main line-guided sprite routine ($01D74D)
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

lg_main:
LDA #$01
%SubOffScreen()

LDA !1540,x; if the move timer is set...
BNE RunStatePtr; skip the next check
LDA $9D; if sprites are locked...
ORA !1626,x; or the stationary flag is set...
BNE draw_smoke_return; return

RunStatePtr:;

LDA !C2,x; sprite state
JSL $0086DF; 16-bit pointer routine

dw State00;
dw State01;
dw State02;


lg_x_offset_lo:
        db $FC,$04,$FC,$04
lg_x_offset_hi:
        db $FF,$00,$FF,$00
lg_y_offset_lo:
        db $FC,$FC,$04,$04
lg_y_offset_hi:
        db $FF,$FF,$00,$00

lg_behavior_tbl_1:
        db $15,$15,$15,$15,$0C,$10,$10,$10
        db $10,$0C,$0C,$10,$10,$10,$10,$0C
        db $15,$15,$10,$10,$10,$10,$10,$10
        db $10,$10,$10,$10,$10,$10,$15,$15
lg_behavior_tbl_2:
        db $00,$00,$00,$00,$00,$00,$01,$02
        db $00,$00,$00,$00,$02,$01,$00,$00
        db $00,$00,$01,$02,$01,$02,$00,$00
        db $00,$00,$02,$02,$00,$00,$00,$00

lg_speed_tbl_y:
        db $00,$10,$00,$F0,$F4,$FC,$F0,$10
        db $04,$0C,$0C,$00,$10,$F0,$FC,$F4
        db $F0,$10,$F0,$10,$F0,$10,$F8,$F8
        db $08,$08,$10,$10,$00,$00,$F0,$10
        db $10,$00,$F0,$F0,$0C,$04,$10,$F0
        db $00,$F4,$F4,$FC,$F0,$10,$00,$0C
        db $10,$F0,$10,$00,$10,$F0,$08,$08
        db $F8,$F8,$F0,$F0,$00,$00,$10,$F0
lg_speed_tbl_x:
        db $10,$00,$10,$00,$0C,$10,$04,$00
        db $10,$0C,$0C,$10,$04,$00,$10,$0C
        db $10,$10,$08,$08,$08,$08,$10,$10
        db $10,$10,$00,$00,$10,$10,$10,$10
        db $00,$F0,$00,$F0,$F4,$F0,$00,$FC
        db $F0,$F4,$F4,$F0,$00,$FC,$F0,$F4
        db $F0,$F0,$F8,$F8,$F8,$F8,$F0,$F0
        db $F0,$F0,$00,$00,$F0,$F0,$F0,$F0

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; code for sprite state 00
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

State00:

LDY #$03;

TileCheckLoop:;

STY $1695|!Base2;

LDA !E4,x;
CLC;
ADC lg_x_offset_lo,y;
STA $02; set up an X position variable, 4 pixels left or right
LDA !14E0,x;
ADC lg_x_offset_hi,y;
STA $03;

LDA !D8,x;
CLC;
ADC lg_y_offset_lo,y;
STA $00; set up a Y position variable, 4 pixels up or down
LDA !14D4,x;
ADC lg_y_offset_hi,y;
STA $01;

LDA !1540,x; if the move timer is set...
BNE GoToPosSet; skip the next part

LDA $00;
AND #$F0; sprite X position with the individual pixel nybble clear
STA $04;
LDA !D8,x;
AND #$F0;
CMP $04; if the sprite Y position is not the same as the X position...
BNE GoToPosSet; go directly to the position setup routine

LDA $02;
AND #$F0; sprite Y position with the individual pixel nybble clear
STA $05;
LDA !E4,x;
AND #$F0;
CMP $05; if the sprite X position is not the same as the Y position...
BEQ DecAndLoop; skip the position setup routine entirely and go to the end of the loop

GoToPosSet:

JSR PositionSetup;
BNE AltIndex;

LDA $1693|!Base2; check the low byte of the "acts like" setting of the Map16 tile that the sprite is touching
CMP #$94;
BEQ OnOffCheck2; if it is 94 or 95...
CMP #$95; then it is an on/off line guide slope
BNE Continue1;

LDA $14AF|!Base2;
BEQ DecAndLoop;
BNE Continue1;

OnOffCheck2:;

LDA $14AF|!Base2;
BNE DecAndLoop;    

Continue1:;

LDA $1693|!Base2;
CMP #$76; if the tile number is less than 76...
BCC DecAndLoop;
CMP #$9A; or greater than 99...
BCC LineGuideTiles; then it is not a line guide tile

DecAndLoop:

LDY $1695|!Base2;
DEY;
BPL TileCheckLoop; loop 4 times

LDA !C2,x; if we're running this code in sprite state 02...
CMP #$02;
BEQ Return01; terminate the code
LDA #$02; if not,
STA !C2,x; set the sprite state to 02

LDY !160E,x; speed index
LDA !157C,x; depending on sprite direction
BEQ NoAddToIndex; if the sprite is moving left...
TYA;
CLC;
ADC #$20; add $20 to the speed index
TAY;
NoAddToIndex:;
LDA lg_speed_tbl_y,y;
BPL $01; if the value is negative...
ASL; left-shift it
PHY;
ASL; left-shift it once more...
STA !AA,x; and store it to the sprite Y speed
PLY;
LDA lg_speed_tbl_x,y;
ASL;
STA !B6,x; set the sprite X speed
LDA #$10;
STA !1540,x; set the time to pause

Return01:;
RTS;

LineGuideTiles:

PHA;
SEC;
SBC #$76; subtract 76 from the tile number so that the index begins at 00
TAY;
PLA; but we still want the actual tile number in A
CMP #$96; if the tile is a line-guide end...
BCC NoAltIndex; 

AltIndex:

LDY !160E,x; then do this
BRA SkipChangePos;

NoAltIndex:;

LDA !D8,x;
STA $08; back up sprite position
LDA !14D4,x;
STA $09;
LDA !E4,x;
STA $0A;
LDA !14E0,x;
STA $0B;

LDA $00; and then set the sprite position
STA !D8,x;
LDA $01; to the offset values from before
STA !14D4,x;
LDA $02;
STA !E4,x;
LDA $03;
STA !14E0,x;

SkipChangePos:

PHB; preserve the data bank
LDA #$07; set the data bank to 07 (or 87)
PHA;
PLB;
LDA $FBF3,y; $07FBF3-$07FC12: low byte of 16-bit pointer to line guide behaviors
STA !151C,x;
LDA $FC13,y; $07FC13-$07FC32: high byte of 16-bit pointer to line guide behaviors
STA !1528,x;
PLB;

LDA lg_behavior_tbl_1,y;
STA !1570,x; not sure what this does

STZ !1534,x;
TYA; save the tile index
STA !160E,x;

LDA !1540,x; if the wait timer is set...
BNE SetState01; change the sprite state to 01

STZ !157C,x; set the sprite direction to right
LDA lg_behavior_tbl_2,y;
BEQ MoreSetups;
TAY;
LDA !D8,x;
CPY #$01;
BNE NoEORPixels;
EOR #$0F;
NoEORPixels:;

BRA SkipLoadX;

MoreSetups:;

LDA !E4,x;

SkipLoadX:

AND #$0F;
CMP #$0A;
BCC NoLeftDir;
LDA !C2,x;
CMP #$02;
BEQ NoLeftDir;
INC !157C,x;
NoLeftDir:;

LDA !D8,x;
STA $0C;
LDA !E4,x;
STA $0D;

JSR State01;

LDA $0C;
SEC;
SBC !D8,x;
CLC;
ADC #$08;
CMP #$10;
BCS RestorePos2;

LDA $0D;
SEC;
SBC !E4,x;
CLC;
ADC #$08;
CMP #$10;
BCS RestorePos2;

SetState01:;

LDA #$01;
STA !C2,x;
RTS;

RestorePos2:

LDA $08;
STA !D8,x; set the sprite position
LDA $09; to the values we stored before
STA !14D4,x;
LDA $0A;
STA !E4,x;
LDA $0B;
STA !14E0,x;  

JMP DecAndLoop

PositionSetup:; Some Map16-checking stuff.  I'm not even going to bother trying to comment this.

LDA $00;
AND #$F0;
STA $06;
LDA $02;
LSR #4;
PHA;
ORA $06;
PHA;
LDA $5B;
AND #$01;
BEQ Map16HLevel;

Map16VLevel:;

PLA;
LDX $01;
CLC;
ADC $00BA80,x;
STA $05;
LDA $00BABC,x;
ADC $03;
STA $06;
BRA Map16Continue;

Map16HLevel:

PLA;
LDX $03;
CLC;
ADC $00BA60,x;
STA $05;
LDA $00BA9C,x;
ADC $01;
STA $06;

Map16Continue:

if !SA1
LDA #$40; bank byte of pointer to tile low byte = $7E
else
LDA #$7E; bank byte of pointer to tile low byte = $7E
endif
STA $07;
LDX $15E9|!Base2;
LDA [$05];
STA $1693|!Base2;
INC $07; bank byte of pointer to tile low byte = $7F
LDA [$05];
PLY;
STY $05;
PHA;
LDA $05;
AND #$07;
TAY;
PLA;
AND bit_mask,y; $018000
RTS

bit_mask:
        db $80,$40,$20,$10,$08,$04,$02,$01

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; code for sprite state 01
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

State01:

LDA $9D;
BNE Return02; return if sprites are locked
LDA !157C,x;
BNE State01Left; run a slightly different routine if the sprite is facing left

State01Right:

LDY !1534,x;

JSR MoveSprPos;

INC !1534,x;
LDA !187B,x; if this sprite table is set...
BEQ SkipFrameChk1;
LDA $13;
LSR;
BCC SkipFrameChk1; then the frame counter increments again every other frame
INC !1534,x;
SkipFrameChk1:;

LDA !1534,x;
CMP !1570,x; if the first counter equals the second...
BCC Return02;
STZ !C2,x; reset the sprite state

Return02:;
RTS

State01Left:;

LDY !1570,x;
DEY;

JSR MoveSprPos;

DEC !1570,x; if this counter has reached 0...
BEQ SetState00; set the sprite state to 00
LDA !187B,x;
BEQ Return02; if this sprite table is set...
LDA $13;
LSR;
BCC Return02; then the counter decrements again every other frame
DEC !1570,x;
BNE Return02;

SetState00:;

STZ !C2,x;
RTS;

MoveSprPos:;

PHB;
LDA #$07; once again, the data bank should be 07/87
PHA;
PLB;
LDA !151C,x; low byte of pointer
STA $04;
LDA !1528,x; high byte of pointer
STA $05;

LDA ($04),y;
AND #$0F; low byte of the pointed-to address: amount to move the sprite on the X-axis
STA $06;
LDA ($04),y;
PLB;
LSR #4;
STA $07;

LDA !D8,x;
AND #$F0;
CLC;
ADC $07; change the sprite's Y position
STA !D8,x;

LDA !E4,x;
AND #$F0;
CLC;
ADC $06; change the sprite's X position
STA !E4,x;

RTS;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; code for sprite state 02
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

State02:

LDA $9D; if spites are locked...
BNE Return03; return

JSL $01802A; update sprite position

LDA !1540,x; if the wait timer is set...
BNE Return03;
LDA !AA,x;
CMP #$20; or the sprite speed is less than 20 (actually, between A0 and 1F)...
BMI Return03;

JSR State00; return

Return03:;
RTS;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; graphics routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

draw_sprite:

%GetDrawInfo()

LDA $00;
SEC;
SBC #$08; X offset - 8
STA $00;

LDA $01;
SEC;
SBC #$08; Y offset - 8
STA $01;

TXA;
ASL #2;
EOR $14;
LSR #3;
AND #$03;
STA $02; index to the motor tiles

;LDA #$05; if the rope is short, draw 5 tiles
STZ $04;
LDA !1510,x;
BEQ ShortRopeTilesNum
LDA #$09; if the rope is long, draw 9 tiles
BRA SetTileCount
ShortRopeTilesNum:
LDA #$05; if the rope is short, draw 5 tiles
SetTileCount:;
STA $03;

LDY !15EA,x; load the OAM index back into Y
LDX #$00; start X off at 00   

GFXLoop:

LDA $00;
STA $0300|!Base2,y; no X displacement
CLC;
ADC #$10; offset the next tile by $10 pixels
STA $00
LDA $01; or Y displacement
STA $0301|!Base2,y; for the motor tile;

LDA rope_tilemap,x;
CPX #$00; if the tilemap index is 0...
BNE UseRopeTiles;
PHX;
LDX $02;
LDA motor_tilemap,x; then use the motor tilemap
PLX;
UseRopeTiles:; else, use the rope tilemap
STA $0302|!Base2,y; set the tile number

LDA #!motor_props;
CPX #$01; if the tilemap index is 0...
BCC SetTileProp; use the tile properties of the motor
LDA #!rope_props; else, use the tile properties of the rope
SetTileProp:;
STA $0303|!Base2,y; set the tile properties

INY #4; increment the OAM index
INX; and the tilemap index
CPX $03; if we have reached our tile count...
BNE GFXLoop; break the loop

LDA #!bottom_rope_tile; set the tile number for the bottom of the rope
STA $02FE|!Base2,y;

LDX $15E9|!Base2; sprite index back into X
LDA #$04; if the rope is short, 5 tiles were drawn
LDY !1510,x;
BEQ SetTilesDrawn;
LDA #$08; if the rope is long, 9 tiles were drawn
SetTilesDrawn:;
LDY #$02; the tiles were all 16x16
JSL $01B7B3;
RTS;
